﻿#include "pch.h"
#include "CenteredGlyphDumpMain.h"

#include <DirectXColors.h>			// For named colors	
#include "Common\DirectXHelper.h"	// For ThrowIfFailed

using namespace CenteredGlyphDump;

using namespace Microsoft::WRL;
using namespace Platform;

// Loads and initializes application assets when the application is loaded.
CenteredGlyphDumpMain::CenteredGlyphDumpMain(const std::shared_ptr<DeviceResources>& deviceResources) :
    m_deviceResources(deviceResources)
{
    m_glyphDumpRenderer = std::unique_ptr<CenteredGlyphDumpRenderer>(new CenteredGlyphDumpRenderer(m_deviceResources));
    CreateWindowSizeDependentResources();

    // TODO: Change timer settings if you want something other than the default variable timestep mode.
    // eg. for 60 FPS fixed timestep update logic, call:
    /*
    m_timer.SetFixedTimeStep(true);
    m_timer.SetTargetElapsedSeconds(1.0 / 60);
    */
    m_deviceResources->RegisterDeviceNotify(this);
}

CenteredGlyphDumpMain::~CenteredGlyphDumpMain()
{
    // Deregister from device notification
    m_deviceResources->RegisterDeviceNotify(nullptr);
}

// Updates application state when the window size changes (e.g. device orientation change)
void CenteredGlyphDumpMain::CreateWindowSizeDependentResources()
{
    m_glyphDumpRenderer->CreateWindowSizeDependentResources();
}

// Updates the application state once per frame.
void CenteredGlyphDumpMain::Update()
{
    // Update scene objects.
    m_timer.Tick([&]()
    {
        // TODO: Replace this with your app content update functions.
        m_glyphDumpRenderer->Update(m_timer);
    });
}

// Renders the current frame according to the current application state.
// Returns true if the frame was rendered and is ready to be displayed.
bool CenteredGlyphDumpMain::Render()
{
    // Don't try to render anything before the first Update.
    if (m_timer.GetFrameCount() == 0)
    {
        return false;
    }

    return m_glyphDumpRenderer->Render();
}

// Notifies renderers that device resources need to be released.
void CenteredGlyphDumpMain::OnDeviceLost()
{
    m_glyphDumpRenderer->ReleaseDeviceDependentResources();
}

// Notifies renderers that device resources may now be recreated.
void CenteredGlyphDumpMain::OnDeviceRecreated()
{
    m_glyphDumpRenderer->CreateDeviceDependentResources();
    CreateWindowSizeDependentResources();
}

void CenteredGlyphDumpMain::SetFontFileKey(ComPtr<IDWriteFontFileLoader> fontFileLoader, String^ filepath)
{
    m_glyphDumpRenderer->SetFontFileKey(fontFileLoader, filepath);
}
