#pragma once

#include <string>
#include "..\DeviceResources.h"
#include "..\Common\StepTimer.h"

namespace CenteredGlyphDump
{
    class CenteredGlyphDumpRenderer
    {
    public:
        CenteredGlyphDumpRenderer(const std::shared_ptr<DeviceResources>& deviceResources);
        void CreateDeviceDependentResources();
        void ReleaseDeviceDependentResources();
        void CreateWindowSizeDependentResources();
        void Update(DX::StepTimer const& timer);
        bool Render();                              // Note: changed to return bool

        // Custom method called from CenteredGlyphDumpMain
        void SetFontFileKey(Microsoft::WRL::ComPtr<IDWriteFontFileLoader> fontFileLoader, Platform::String^ key);

    private:
        // Cached pointer to device resources.
        std::shared_ptr<DeviceResources> m_deviceResources;
        bool m_needsRedraw;

        Microsoft::WRL::ComPtr<ID2D1DrawingStateBlock>  m_stateBlock;
        Microsoft::WRL::ComPtr<IDWriteFontFace>         m_fontFace;
        Microsoft::WRL::ComPtr<ID2D1SolidColorBrush>    m_blackBrush;

        DWRITE_GLYPH_RUN                                m_glyphRun;
        std::vector<uint16>                             m_glyphIndices;
        std::vector<float>                              m_glyphAdvances;
        std::vector<DWRITE_GLYPH_OFFSET>                m_glyphOffsets;

        void DefineNewGlyphRun();
    };
}