#include "pch.h"
#include "PrivateFontFileStream.h"

PrivateFontFileStream::PrivateFontFileStream(Platform::Array<byte>^ fileBuffer) : m_refCount(0),
                                                                                  m_fileBuffer(fileBuffer)
{
}

// IUnknown methods
ULONG STDMETHODCALLTYPE PrivateFontFileStream::AddRef()
{
    m_refCount++;
    return m_refCount;
}

ULONG STDMETHODCALLTYPE PrivateFontFileStream::Release()
{
    m_refCount--;
    LONG newCount = m_refCount;

    if (m_refCount == 0)
        delete this;

    return newCount;
}

HRESULT STDMETHODCALLTYPE PrivateFontFileStream::QueryInterface(_In_ REFIID riid, _Outptr_ void** ppOutput)
{
    *ppOutput = nullptr;
    HRESULT hr = S_OK;

    if (riid == __uuidof(IDWriteFontFileStream))
    {
        *ppOutput = static_cast<IDWriteFontFileStream*>(this);
        AddRef();
    }
    else if (riid == __uuidof(IUnknown))
    {
        *ppOutput = this;
        AddRef();
    }
    else
    {
        hr = E_NOINTERFACE;
    }
    return hr;
}

HRESULT STDMETHODCALLTYPE PrivateFontFileStream::GetFileSize(_Out_ UINT64 * fileSize)
{
    *fileSize = m_fileBuffer->Length;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE PrivateFontFileStream::GetLastWriteTime(_Out_ UINT64 * lastWriteTime)
{
    // This is more useful for font files read off disk 
    //  to avoid reloading unchanged files.
    *lastWriteTime = 0;
    return E_NOTIMPL;
}

HRESULT STDMETHODCALLTYPE PrivateFontFileStream::ReadFileFragment(_Out_ const void ** fragmentStart,
                                                                  UINT64 fileOffset,
                                                                  UINT64 fragmentSize,
                                                                  _Out_ void ** fragmentContext)
{
    // No memory must be allocated for ReadFileFragment,
    //  so nothing needs to be released later.
    // fragmentContext argument is not used.
    *fragmentContext = nullptr;

    if (fileOffset + fragmentSize > m_fileBuffer->Length)
    {
        *fragmentStart = nullptr;
        return E_INVALIDARG;
    }

    *fragmentStart = m_fileBuffer->Data + fileOffset;
    return S_OK;
}
                                           
void STDMETHODCALLTYPE PrivateFontFileStream::ReleaseFileFragment(void * fragmentContext)
{
    // No memory has been allocated for ReadFileFragment,
    //  so nothing needs to be released here.
}
