﻿#include "pch.h"
#include "GlyphDumpMain.h"
#include "Common\DirectXHelper.h"	// For ThrowIfFailed

using namespace GlyphDump;

using namespace Platform;

// Loads and initializes application assets when the application is loaded.
GlyphDumpMain::GlyphDumpMain(const std::shared_ptr<DeviceResources>& deviceResources) :
	m_deviceResources(deviceResources)
{
	m_glyphDumpRenderer = std::unique_ptr<GlyphDumpRenderer>(new GlyphDumpRenderer(m_deviceResources));
    CreateWindowSizeDependentResources();

	// TODO: Change timer settings if you want something other than the default variable timestep mode.
	// eg. for 60 FPS fixed timestep update logic, call:
	/*
	m_timer.SetFixedTimeStep(true);
	m_timer.SetTargetElapsedSeconds(1.0 / 60);
	*/
	m_deviceResources->RegisterDeviceNotify(this);
}

GlyphDumpMain::~GlyphDumpMain()
{
	// Deregister from device notification
	m_deviceResources->RegisterDeviceNotify(nullptr);
}

// Updates application state when the window size changes (e.g. device orientation change)
void GlyphDumpMain::CreateWindowSizeDependentResources() 
{
    m_glyphDumpRenderer->CreateWindowSizeDependentResources();
}

// Updates the application state once per frame.
void GlyphDumpMain::Update() 
{
	// Update scene objects.
	m_timer.Tick([&]()
	{
		// TODO: Replace this with your app content update functions.
		m_glyphDumpRenderer->Update(m_timer);
	});
}

// Renders the current frame according to the current application state.
// Returns true if the frame was rendered and is ready to be displayed.
bool GlyphDumpMain::Render() 
{
	// Don't try to render anything before the first Update.
	if (m_timer.GetFrameCount() == 0)
	{
		return false;
	}

	return m_glyphDumpRenderer->Render();
}

// Notifies renderers that device resources need to be released.
void GlyphDumpMain::OnDeviceLost()
{
	m_glyphDumpRenderer->ReleaseDeviceDependentResources();
}

// Notifies renderers that device resources may now be recreated.
void GlyphDumpMain::OnDeviceRecreated()
{
	m_glyphDumpRenderer->CreateDeviceDependentResources();
	CreateWindowSizeDependentResources();
}

void GlyphDumpMain::SetFontFilePath(String^ filepath)
{
    m_glyphDumpRenderer->SetFontFilePath(std::wstring(filepath->Data()));
}
