﻿#include "pch.h"
#include "HelloGlyphRunMain.h"

#include <DirectXColors.h>			// For named colors	
#include "Common\DirectXHelper.h"	// For ThrowIfFailed

using namespace HelloGlyphRun;

// Loads and initializes application assets when the application is loaded.
HelloGlyphRunMain::HelloGlyphRunMain(const std::shared_ptr<DeviceResources>& deviceResources) :
	m_deviceResources(deviceResources)
{
	m_glyphRunRenderer = std::unique_ptr<HelloGlyphRunRenderer>(new HelloGlyphRunRenderer(m_deviceResources));
    CreateWindowSizeDependentResources();

	// TODO: Change timer settings if you want something other than the default variable timestep mode.
	// eg. for 60 FPS fixed timestep update logic, call:
	/*
	m_timer.SetFixedTimeStep(true);
	m_timer.SetTargetElapsedSeconds(1.0 / 60);
	*/
	m_deviceResources->RegisterDeviceNotify(this);
}

HelloGlyphRunMain::~HelloGlyphRunMain()
{
	// Deregister from device notification
	m_deviceResources->RegisterDeviceNotify(nullptr);
}

// Updates application state when the window size changes (e.g. device orientation change)
void HelloGlyphRunMain::CreateWindowSizeDependentResources() 
{
	m_glyphRunRenderer->CreateWindowSizeDependentResources();
}

// Updates the application state once per frame.
void HelloGlyphRunMain::Update() 
{
	// Update scene objects.
	m_timer.Tick([&]()
	{
		m_glyphRunRenderer->Update(m_timer);
	});
}

// Renders the current frame according to the current application state.
// Returns true if the frame was rendered and is ready to be displayed.
bool HelloGlyphRunMain::Render() 
{
	// Don't try to render anything before the first Update.
	if (m_timer.GetFrameCount() == 0)
	{
		return false;
	}

	m_glyphRunRenderer->Render();

	return true;
}

// Notifies renderers that device resources need to be released.
void HelloGlyphRunMain::OnDeviceLost()
{
	m_glyphRunRenderer->ReleaseDeviceDependentResources();
}

// Notifies renderers that device resources may now be recreated.
void HelloGlyphRunMain::OnDeviceRecreated()
{
	m_glyphRunRenderer->CreateDeviceDependentResources();
	CreateWindowSizeDependentResources();
}
