﻿#include "pch.h"
#include "OutlinedCharactersMain.h"

#include <DirectXColors.h>			// For named colors	
#include "Common\DirectXHelper.h"	// For ThrowIfFailed

using namespace OutlinedCharacters;

using namespace Microsoft::WRL;

// Loads and initializes application assets when the application is loaded.
OutlinedCharactersMain::OutlinedCharactersMain(const std::shared_ptr<DeviceResources>& deviceResources) :
	m_deviceResources(deviceResources)
{
	m_outlinedCharactersRenderer = std::unique_ptr<OutlinedCharactersRenderer>(new OutlinedCharactersRenderer(m_deviceResources));
    m_outlinedCharactersRenderer->CreateWindowSizeDependentResources();

	// TODO: Change timer settings if you want something other than the default variable timestep mode.
	// eg. for 60 FPS fixed timestep update logic, call:
	/*
	m_timer.SetFixedTimeStep(true);
	m_timer.SetTargetElapsedSeconds(1.0 / 60);
	*/
	m_deviceResources->RegisterDeviceNotify(this);
}

OutlinedCharactersMain::~OutlinedCharactersMain()
{
	// Deregister from device notification
	m_deviceResources->RegisterDeviceNotify(nullptr);
}

// Updates application state when the window size changes (e.g. device orientation change)
void OutlinedCharactersMain::CreateWindowSizeDependentResources() 
{
    m_outlinedCharactersRenderer->CreateWindowSizeDependentResources();
}

// Updates the application state once per frame.
void OutlinedCharactersMain::Update() 
{
	// Update scene objects.
	m_timer.Tick([&]()
	{
        m_outlinedCharactersRenderer->Update(m_timer);
	});
}

// Renders the current frame according to the current application state.
// Returns true if the frame was rendered and is ready to be displayed.
bool OutlinedCharactersMain::Render() 
{
	// Don't try to render anything before the first Update.
	if (m_timer.GetFrameCount() == 0)
	{
		return false;
	}

    m_outlinedCharactersRenderer->Render();

	return true;
}

// Notifies renderers that device resources need to be released.
void OutlinedCharactersMain::OnDeviceLost()
{
    m_outlinedCharactersRenderer->ReleaseDeviceDependentResources();
}

// Notifies renderers that device resources may now be recreated.
void OutlinedCharactersMain::OnDeviceRecreated()
{
    m_outlinedCharactersRenderer->CreateDeviceDependentResources();
	CreateWindowSizeDependentResources();
}

void OutlinedCharactersMain::SetFontFace(ComPtr<IDWriteFontFace> fontFace)
{
    m_outlinedCharactersRenderer->SetFontFace(fontFace);
}
