#include "pch.h"
#include "PrivateFontCollectionLoader.h"
#include "PrivateFontEnumerator.h"

using namespace Concurrency;
using namespace Microsoft::WRL;
using namespace Platform;
using namespace Windows::Storage;

// The single instance of this class
ComPtr<PrivateFontCollectionLoader> PrivateFontCollectionLoader::m_fontCollectionLoader = nullptr;

// Private constructor
PrivateFontCollectionLoader::PrivateFontCollectionLoader(IDWriteFactory * dwriteFactory) : m_refCount(0)
{
    m_fontEnumerator = new PrivateFontEnumerator(dwriteFactory);
}

// IUnknown methods
ULONG STDMETHODCALLTYPE PrivateFontCollectionLoader::AddRef()
{
    m_refCount++;
    return m_refCount;
}

ULONG STDMETHODCALLTYPE PrivateFontCollectionLoader::Release()
{
    m_refCount--;
    LONG newCount = m_refCount;

    if (m_refCount == 0)
        delete this;

    return newCount;
}

HRESULT STDMETHODCALLTYPE PrivateFontCollectionLoader::QueryInterface(_In_ REFIID riid, _Outptr_ void** ppOutput)
{
    *ppOutput = nullptr;
    HRESULT hr = S_OK;

    if (riid == __uuidof(IDWriteFontCollection))
    {
        *ppOutput = static_cast<IDWriteFontCollectionLoader*>(this);
        AddRef();
    }
    else if (riid == __uuidof(IUnknown))
    {
        *ppOutput = this;
        AddRef();
    }
    else
    {
        hr = E_NOINTERFACE;
    }
    return hr;
}

// Static public creation method to implement singleton model
PrivateFontCollectionLoader * PrivateFontCollectionLoader::Create(IDWriteFactory * dwriteFactory)
{
    if (m_fontCollectionLoader == nullptr)
        m_fontCollectionLoader = new PrivateFontCollectionLoader(dwriteFactory);

    return m_fontCollectionLoader.Get();
}

// Public asynchronous method
task<void> PrivateFontCollectionLoader::LoadFontsAsync(StorageFolder^ storageFolder, String^ subDirectory)
{
    return m_fontEnumerator->LoadFontsAsync(storageFolder, subDirectory);
}

HRESULT STDMETHODCALLTYPE PrivateFontCollectionLoader::CreateEnumeratorFromKey(IDWriteFactory * factory,
                                                                               _In_   const void * collectionKey,
                                                                               UINT32  collectionKeySize,
                                                                               _Out_  IDWriteFontFileEnumerator ** fontFileEnumerator)
{
    // There's only one font enumerator, so the key isn't used
    *fontFileEnumerator = m_fontEnumerator.Get();

    return S_OK;
}

