#include "pch.h"
#include "PrivateFontEnumerator.h"

using namespace Concurrency;
using namespace Platform;
using namespace Windows::Storage;

PrivateFontEnumerator::PrivateFontEnumerator(IDWriteFactory * writeFactory) : m_refCount(0),
                                                                              m_enumerator(-1),
                                                                              m_writeFactory(writeFactory)
{
    // Instantiate and register FontFileLoader
    m_fontFileLoader = PrivateFontFileLoader::Create();
    writeFactory->RegisterFontFileLoader(m_fontFileLoader.Get());
}

// IUnknown methods
ULONG STDMETHODCALLTYPE PrivateFontEnumerator::AddRef()
{
    m_refCount++;
    return m_refCount;
}

ULONG STDMETHODCALLTYPE PrivateFontEnumerator::Release()
{
    m_refCount--;
    LONG newCount = m_refCount;

    if (m_refCount == 0)
        delete this;

    return newCount;
}

HRESULT STDMETHODCALLTYPE PrivateFontEnumerator::QueryInterface(_In_ REFIID riid, _Outptr_ void** ppOutput)
{
    *ppOutput = nullptr;
    HRESULT hr = S_OK;

    if (riid == __uuidof(IDWriteFontFileEnumerator))
    {
        *ppOutput = static_cast<IDWriteFontFileEnumerator*>(this);
        AddRef();
    }
    else if (riid == __uuidof(IUnknown))
    {
        *ppOutput = this;
        AddRef();
    }
    else
    {
        hr = E_NOINTERFACE;
    }
    return hr;
}

task<void> PrivateFontEnumerator::LoadFontsAsync(StorageFolder^ storageFolder, String^ subDirectory)
{
    // Load fonts in PrivateFontFileLoader
    task<void> loadFontsTask = create_task(m_fontFileLoader->LoadFontsAsync(storageFolder, subDirectory));
    return loadFontsTask.then([this]()
    {
        m_fontFileKeys = m_fontFileLoader->GetFileNames();
    });
}

HRESULT STDMETHODCALLTYPE PrivateFontEnumerator::GetCurrentFontFile(_Out_ IDWriteFontFile ** fontFile)
{
    if (m_enumerator < 0 || m_enumerator >= m_fontFileKeys.size())
        return E_NOT_VALID_STATE;

    String^ key = m_fontFileKeys[m_enumerator];
    return m_writeFactory->CreateCustomFontFileReference(&key, sizeof(String^), m_fontFileLoader.Get(), fontFile);
}

HRESULT STDMETHODCALLTYPE PrivateFontEnumerator::MoveNext(_Out_ BOOL * hasCurrentFile)
{
    m_enumerator++;
    *hasCurrentFile = m_enumerator < m_fontFileKeys.size();
    return S_OK;
}
