﻿#include "pch.h"
#include "ResourceFontLayoutMain.h"

#include <DirectXColors.h>			// For named colors	
#include "Common\DirectXHelper.h"	// For ThrowIfFailed

using namespace ResourceFontLayout;

using namespace Microsoft::WRL;
using namespace Platform;

// Loads and initializes application assets when the application is loaded.
ResourceFontLayoutMain::ResourceFontLayoutMain(const std::shared_ptr<DeviceResources>& deviceResources) :
	m_deviceResources(deviceResources)
{
	m_ResourceFontLayoutRenderer = std::unique_ptr<ResourceFontLayoutRenderer>(new ResourceFontLayoutRenderer(m_deviceResources));
    CreateWindowSizeDependentResources();

	// TODO: Change timer settings if you want something other than the default variable timestep mode.
	// eg. for 60 FPS fixed timestep update logic, call:
	/*
	m_timer.SetFixedTimeStep(true);
	m_timer.SetTargetElapsedSeconds(1.0 / 60);
	*/
	m_deviceResources->RegisterDeviceNotify(this);
}

ResourceFontLayoutMain::~ResourceFontLayoutMain()
{
	// Deregister from device notification
	m_deviceResources->RegisterDeviceNotify(nullptr);
}

// Updates application state when the window size changes (e.g. device orientation change)
void ResourceFontLayoutMain::CreateWindowSizeDependentResources() 
{
	m_ResourceFontLayoutRenderer->CreateWindowSizeDependentResources();
}

// Updates the application state once per frame.
void ResourceFontLayoutMain::Update() 
{
	// Update scene objects.
	m_timer.Tick([&]()
	{
		m_ResourceFontLayoutRenderer->Update(m_timer);
	});
}

// Renders the current frame according to the current application state.
// Returns true if the frame was rendered and is ready to be displayed.
bool ResourceFontLayoutMain::Render() 
{
	// Don't try to render anything before the first Update.
	if (m_timer.GetFrameCount() == 0)
	{
		return false;
	}

    return m_ResourceFontLayoutRenderer->Render();
}

// Notifies renderers that device resources need to be released.
void ResourceFontLayoutMain::OnDeviceLost()
{
	m_ResourceFontLayoutRenderer->ReleaseDeviceDependentResources();
}

// Notifies renderers that device resources may now be recreated.
void ResourceFontLayoutMain::OnDeviceRecreated()
{
	m_ResourceFontLayoutRenderer->CreateDeviceDependentResources();
	CreateWindowSizeDependentResources();
}

void ResourceFontLayoutMain::SetFontFamilyName(ComPtr<IDWriteFontCollection> fontCollection, String^ familyName)
{
    m_ResourceFontLayoutRenderer->SetFontFamilyName(fontCollection, std::wstring(familyName->Data()));
}

