﻿//
// MainPage.xaml.cpp
// Implementation of the MainPage class.
//

#include "pch.h"
#include "DirectXPage.xaml.h"
#include "Common\DirectXHelper.h"

using namespace SystemFontGlyphs;

using namespace Microsoft::WRL;
using namespace Platform;
using namespace Platform::Collections;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::Graphics::Display;
using namespace Windows::System::Threading;
using namespace Windows::UI::Core;
using namespace Windows::UI::Input;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

DirectXPage::DirectXPage():
	m_windowVisible(true)
{
	InitializeComponent();

	// Register event handlers for page lifecycle.
	CoreWindow^ window = Window::Current->CoreWindow;

	window->VisibilityChanged +=
		ref new TypedEventHandler<CoreWindow^, VisibilityChangedEventArgs^>(this, &DirectXPage::OnVisibilityChanged);

	DisplayInformation^ currentDisplayInformation = DisplayInformation::GetForCurrentView();

	currentDisplayInformation->DpiChanged +=
		ref new TypedEventHandler<DisplayInformation^, Object^>(this, &DirectXPage::OnDpiChanged);

	currentDisplayInformation->OrientationChanged +=
		ref new TypedEventHandler<DisplayInformation^, Object^>(this, &DirectXPage::OnOrientationChanged);

	DisplayInformation::DisplayContentsInvalidated +=
		ref new TypedEventHandler<DisplayInformation^, Object^>(this, &DirectXPage::OnDisplayContentsInvalidated);

	swapChainPanel->CompositionScaleChanged += 
		ref new TypedEventHandler<SwapChainPanel^, Object^>(this, &DirectXPage::OnCompositionScaleChanged);

	swapChainPanel->SizeChanged +=
		ref new SizeChangedEventHandler(this, &DirectXPage::OnSwapChainPanelSizeChanged);


	// Register the rendering event, called every time XAML renders the screen.
	m_eventToken = CompositionTarget::Rendering::add(ref new EventHandler<Object^>(this, &DirectXPage::OnRendering));

	// Whenever this screen is not being used anymore, you can unregister this event with the following line:
	// CompositionTarget::Rendering::remove(m_eventToken);

	// Disable all pointer visual feedback for better performance when touching.
	auto pointerVisualizationSettings = PointerVisualizationSettings::GetForCurrentView();
	pointerVisualizationSettings->IsContactFeedbackEnabled = false; 
	pointerVisualizationSettings->IsBarrelButtonFeedbackEnabled = false;

	// At this point we have access to the device. 
	// We can create the device-dependent resources.
	m_deviceResources = std::make_shared<DeviceResources>();
	m_deviceResources->SetWindow(Window::Current->CoreWindow, swapChainPanel);

	// Register our SwapChainPanel to get independent input pointer events
	auto workItemHandler = ref new WorkItemHandler([this] (IAsyncAction ^)
	{
		// The CoreIndependentInputSource will raise pointer events for the specified device types on whichever thread it's created on.
		CoreIndependentInputSource^ coreInput = swapChainPanel->CreateCoreIndependentInputSource(
			Windows::UI::Core::CoreInputDeviceTypes::Mouse |
			Windows::UI::Core::CoreInputDeviceTypes::Touch |
			Windows::UI::Core::CoreInputDeviceTypes::Pen
			);

		// Register for pointer events, which will be raised on the background thread.
		coreInput->PointerPressed += ref new TypedEventHandler<Object^, PointerEventArgs^>(this, &DirectXPage::OnPointerPressed);
		coreInput->PointerMoved += ref new TypedEventHandler<Object^, PointerEventArgs^>(this, &DirectXPage::OnPointerMoved);
		coreInput->PointerReleased += ref new TypedEventHandler<Object^, PointerEventArgs^>(this, &DirectXPage::OnPointerReleased);

		// Begin processing input messages as they're delivered.
		coreInput->Dispatcher->ProcessEvents(CoreProcessEventsOption::ProcessUntilQuit);
	});

	// Run task on a dedicated high priority background thread.
	m_inputLoopWorker = ThreadPool::RunAsync(workItemHandler, WorkItemPriority::High, WorkItemOptions::TimeSliced);


	m_main = std::unique_ptr<SystemFontGlyphsMain>(new SystemFontGlyphsMain(m_deviceResources));

    // Get system font collection and font families
    DX::ThrowIfFailed(
        m_deviceResources->GetDWriteFactory()->GetSystemFontCollection(&m_systemFontCollection, true)
        );

    // Set ItemsSource of ListBox to system font families
    lstbox->ItemsSource = GetFontFamilies(m_systemFontCollection);
}

// Saves the current state of the app for suspend and terminate events.
void DirectXPage::SaveInternalState(IPropertySet^ state)
{
	m_deviceResources->Trim();

	// Put code to save app state here.
}

// Loads the current state of the app for resume events.
void DirectXPage::LoadInternalState(IPropertySet^ state)
{
	// Put code to load app state here.
}

// Called every time XAML decides to render a frame.
void DirectXPage::OnRendering(Object^ sender, Object^ args)
{
	if (m_windowVisible)
	{
		m_main->Update();

		if (m_main->Render())
		{
			m_deviceResources->Present();
		}
	}
}

// Window event handlers.

void DirectXPage::OnVisibilityChanged(CoreWindow^ sender, VisibilityChangedEventArgs^ args)
{
	m_windowVisible = args->Visible;
}

// Display properties event handlers.

void DirectXPage::OnDpiChanged(DisplayInformation^ sender, Object^ args)
{
	m_deviceResources->SetDpi(sender->LogicalDpi);
}


void DirectXPage::OnOrientationChanged(DisplayInformation^ sender, Object^ args)
{
	m_deviceResources->UpdateForWindowSizeChange();
	m_main->CreateWindowSizeDependentResources();
}


void DirectXPage::OnDisplayContentsInvalidated(DisplayInformation^ sender, Object^ args)
{
	m_deviceResources->ValidateDevice();
}

void DirectXPage::OnPointerPressed(Object^ sender, PointerEventArgs^ e)
{
	// Handle the PointerPressed event, which will be raised on a background thread.
}

void DirectXPage::OnPointerMoved(Object^ sender, PointerEventArgs^ e)
{
	// Handle the PointerMoved event, which will be raised on a background thread.
}

void DirectXPage::OnPointerReleased(Object^ sender, PointerEventArgs^ e)
{
	// Handle the PointerReleased event, which will be raised on a background thread.
}

void DirectXPage::OnCompositionScaleChanged(SwapChainPanel^ sender, Object^ args)
{
	m_deviceResources->CreateWindowSizeDependentResources();
	m_main->CreateWindowSizeDependentResources();
}

void DirectXPage::OnSwapChainPanelSizeChanged(Object^ sender, SizeChangedEventArgs^ e)
{
    m_deviceResources->UpdateForWindowSizeChange();
	m_main->CreateWindowSizeDependentResources();
}

// Called from constructor to fill up ListBox
IVector<String^>^ DirectXPage::GetFontFamilies(ComPtr<IDWriteFontCollection> fontCollection)
{
    unsigned int count = fontCollection->GetFontFamilyCount();
    std::vector<String^> fonts(count);

    // Loop through the font families
    for (unsigned int i = 0; i < count; i++)
    {
        ComPtr<IDWriteFontFamily> fontFamily;

        DX::ThrowIfFailed(
            fontCollection->GetFontFamily(i, &fontFamily)
            );

        ComPtr<IDWriteLocalizedStrings> familyNames;

        DX::ThrowIfFailed(
            fontFamily->GetFamilyNames(&familyNames)
            );

        unsigned int index = 0;
        BOOL exists;

        DX::ThrowIfFailed(
            familyNames->FindLocaleName(L"en-us", &index, &exists)
            );

        if (exists)
        {
            unsigned int length = 0;

            DX::ThrowIfFailed(
                familyNames->GetStringLength(index, &length)
                );

            std::vector<wchar_t> str(length + 1);

            DX::ThrowIfFailed(
                familyNames->GetString(index, str.data(), length + 1)
                );

            fonts[i] = ref new String(str.data());
        }
    }
    return ref new Vector<String^>(fonts.data(), fonts.size());
}

void DirectXPage::OnListBoxSelectionChanged(Object^ sender, SelectionChangedEventArgs^ args)
{
    // Get the selected family name
    ListBox^ lstbox = safe_cast<ListBox^>(sender);
    String^ strFontFamily = dynamic_cast<String^>(lstbox->SelectedItem);

    if (strFontFamily != nullptr)
    {
        // Find the family name
        unsigned int index;
        BOOL exists;

        DX::ThrowIfFailed(
            m_systemFontCollection->FindFamilyName(strFontFamily->Data(), &index, &exists)
            );

        // Get the font family
        ComPtr<IDWriteFontFamily> fontFamily;

        DX::ThrowIfFailed(
            m_systemFontCollection->GetFontFamily(index, &fontFamily)
            );

        // Get the font
        ComPtr<IDWriteFont> font;

        DX::ThrowIfFailed(
            fontFamily->GetFirstMatchingFont(DWRITE_FONT_WEIGHT::DWRITE_FONT_WEIGHT_NORMAL,
                                             DWRITE_FONT_STRETCH::DWRITE_FONT_STRETCH_NORMAL,
                                             DWRITE_FONT_STYLE::DWRITE_FONT_STYLE_NORMAL,
                                             &font)
            );

        // Create the font face
        ComPtr<IDWriteFontFace> fontFace;

        DX::ThrowIfFailed(
            font->CreateFontFace(&fontFace)
            );

        // Pass it to the renderer
        m_main->SetFontFace(fontFace.Get());
    }
}
