﻿#include "pch.h"
#include "SystemFontGlyphsMain.h"

#include <DirectXColors.h>			// For named colors	
#include "Common\DirectXHelper.h"	// For ThrowIfFailed

using namespace SystemFontGlyphs;

using namespace Microsoft::WRL;

// Loads and initializes application assets when the application is loaded.
SystemFontGlyphsMain::SystemFontGlyphsMain(const std::shared_ptr<DeviceResources>& deviceResources) :
	m_deviceResources(deviceResources)
{
	m_glyphsRenderer = std::unique_ptr<SystemFontGlyphsRenderer>(new SystemFontGlyphsRenderer(m_deviceResources));
    m_glyphsRenderer->CreateWindowSizeDependentResources();

	// TODO: Change timer settings if you want something other than the default variable timestep mode.
	// eg. for 60 FPS fixed timestep update logic, call:
	/*
	m_timer.SetFixedTimeStep(true);
	m_timer.SetTargetElapsedSeconds(1.0 / 60);
	*/
	m_deviceResources->RegisterDeviceNotify(this);
}

SystemFontGlyphsMain::~SystemFontGlyphsMain()
{
	// Deregister from device notification
	m_deviceResources->RegisterDeviceNotify(nullptr);
}

// Updates application state when the window size changes (e.g. device orientation change)
void SystemFontGlyphsMain::CreateWindowSizeDependentResources() 
{
	m_glyphsRenderer->CreateWindowSizeDependentResources();
}

// Updates the application state once per frame.
void SystemFontGlyphsMain::Update() 
{
	// Update scene objects.
	m_timer.Tick([&]()
	{
		m_glyphsRenderer->Update(m_timer);
	});
}

// Renders the current frame according to the current application state.
// Returns true if the frame was rendered and is ready to be displayed.
bool SystemFontGlyphsMain::Render() 
{
	// Don't try to render anything before the first Update.
	if (m_timer.GetFrameCount() == 0)
	{
		return false;
	}

	m_glyphsRenderer->Render();
	return true;
}

// Notifies renderers that device resources need to be released.
void SystemFontGlyphsMain::OnDeviceLost()
{
	m_glyphsRenderer->ReleaseDeviceDependentResources();
}

// Notifies renderers that device resources may now be recreated.
void SystemFontGlyphsMain::OnDeviceRecreated()
{
	m_glyphsRenderer->CreateDeviceDependentResources();
	CreateWindowSizeDependentResources();
}

void SystemFontGlyphsMain::SetFontFace(ComPtr<IDWriteFontFace> fontFace)
{
    m_glyphsRenderer->SetFontFace(fontFace);
}
